@@ -61,7 +61,13 @@ class CommandHandler:
             await bot.respond(event.ctx, command_handler.usage)
 
         except exceptions.TSCommandError as e:
-            bot.emit(event_name="command_error", msg=str(e), ctx=event.ctx)
+            bot.emit(
+                event_name="command_error",
+                ctx={"exception": e.__class__.__name__, "exception_msg": str(e)} | event.ctx,
+            )
 
         except exceptions.TSPermissionError as e:
-            bot.emit(event_name="permission_error", msg=str(e), ctx=event.ctx)
+            bot.emit(
+                event_name="permission_error",
+                ctx={"exception": e.__class__.__name__, "exception_msg": str(e)} | event.ctx,
+            )
