@@ -3,51 +3,31 @@ from __future__ import annotations
 import asyncio
 import inspect
 import itertools
-from typing import TYPE_CHECKING, Any
+from dataclasses import dataclass, field
+from typing import TYPE_CHECKING, Awaitable, Callable
 
 from tsbot import utils
 
 if TYPE_CHECKING:
-    from tsbot import bot, plugin, typealiases
-
-
-def add_check(func: typealiases.TCommandHandler) -> TSCommand:
-    def check_decorator(command_handler: TSCommand) -> TSCommand:
-        command_handler.add_check(func)
-        return command_handler
-
-    return check_decorator  # type: ignore
+    from tsbot import bot, typealiases
 
 
+@dataclass()
 class TSCommand:
-    def __init__(
-        self,
-        commands: tuple[str, ...],
-        handler: typealiases.TCommandHandler | typealiases.TPluginCommandHandler,
-        *,
-        help_text: str | None = None,
-        raw: bool = False,
-        hidden: bool = False,
-    ) -> None:
-        self.commands = commands
-        self.handler = handler
+    commands: tuple[str, ...]
+    handler: Callable[..., Awaitable[None]]
 
-        self.help_text = help_text
-        self.raw = raw
-        self.hidden = hidden
+    help_text: str = field(repr=False)
+    raw: bool = field(repr=False)
+    hidden: bool = field(repr=False)
 
-        self.plugin_instance: plugin.TSPlugin | None = None
-        self.checks: list[typealiases.TCommandHandler] = []
-
-    def add_check(self, func: typealiases.TCommandHandler) -> None:
-        self.checks.append(func)
+    checks: list[Callable[..., Awaitable[None]]] = field(default_factory=list, repr=False)
 
     @property
     def call_signature(self) -> tuple[inspect.Parameter, ...]:
         signature = inspect.signature(self.handler)
-        params_to_discard = 3 if self.plugin_instance else 2
 
-        return tuple(itertools.islice(signature.parameters.values(), params_to_discard, None))
+        return tuple(itertools.islice(signature.parameters.values(), 2, None))
 
     @property
     def usage(self) -> str:
@@ -84,20 +64,9 @@ class TSCommand:
                 raise exception
 
     async def run(self, bot: bot.TSBot, ctx: typealiases.TCtx, msg: str) -> None:
-        kwargs: dict[str, str]
-        args: tuple[str, ...]
-
         args, kwargs = ((msg,), {}) if self.raw else utils.parse_args_kwargs(msg)
 
         if self.checks:
             await self.run_checks(bot, ctx, *args, **kwargs)
 
-        command_args = (bot, ctx) if not self.plugin_instance else (self.plugin_instance, bot, ctx)
-
-        await self.handler(*command_args, *args, **kwargs)  # type: ignore
-
-    def __call__(self, *args: Any, **kwargs: Any):
-        return self.run(*args, **kwargs)
-
-    def __repr__(self) -> str:
-        return f"{self.__class__.__qualname__}(commands={self.commands!r}, handler={self.handler.__qualname__!r})"
+        await self.handler(bot, ctx, *args, **kwargs)
