@@ -42,21 +42,21 @@ class TSQuery:
     def __repr__(self) -> str:
         return f"{self.__class__.__qualname__}({self.command!r})"
 
-    def option(self: T, *args: typealiases.Stringable) -> T:
+    def option(self: T, *args: typealiases.SupportsStr) -> T:
         """Add options to the command eg. ``-groups``"""
         self._dirty = True
 
         self._options.extend(str(arg) for arg in args)
         return self
 
-    def params(self: T, **kwargs: typealiases.Stringable) -> T:
+    def params(self: T, **kwargs: typealiases.SupportsStr) -> T:
         """Add parameters to the command eg. ``cldbid=12``"""
         self._dirty = True
 
         self._parameters.update({k: str(v) for k, v in kwargs.items()})
         return self
 
-    def param_block(self: T, **kwargs: typealiases.Stringable) -> T:
+    def param_block(self: T, **kwargs: typealiases.SupportsStr) -> T:
         """Add parameter blocks eg. ``clid=1 | clid=2 | clid=3`` to the command"""
         self._dirty = True
 
